# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .posted_voice_name import PostedVoiceName
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class PostedVoice(UniversalBaseModel):
    """
    A Voice specification posted to the server
    """

    provider: typing.Literal["HUME_AI"] = pydantic.Field(default="HUME_AI")
    """
    The provider of the voice to use. Currently, only `HUME_AI` is supported as the voice provider.
    """

    name: typing.Optional[PostedVoiceName] = pydantic.Field(default=None)
    """
    String with the name of the voice to use. Maximum length of 75 characters. Will be converted to all-uppercase.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
