# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .return_custom_voice import ReturnCustomVoice
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ReturnPagedCustomVoices(UniversalBaseModel):
    """
    A paginated list of custom voices returned from the server
    """

    page_number: int = pydantic.Field()
    """
    The page number of the returned results.
    """

    page_size: int = pydantic.Field()
    """
    The number of results returned per page.
    """

    total_pages: int = pydantic.Field()
    """
    The total number of pages in the collection
    """

    custom_voices_page: typing.List[ReturnCustomVoice] = pydantic.Field()
    """
    List of custom voices returned for the specified page number and page size.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
