# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class ReturnActiveChatCountPerTag(UniversalBaseModel):
    """
    A description of current chat chat sessions per tag
    """

    tag: str = pydantic.Field()
    """
    User tag applied to a chat.
    """

    total_tag_active_chats: int = pydantic.Field()
    """
    The total number of active chats for this user with the specified tag.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
